#include <iostream>
using namespace std;

#include "capd/capdlib.h"
using namespace capd;
using namespace capd::alglib;
using namespace capd::matrixAlgorithms;

void linalgdemo()
{
	// Matrixes and vectors
	IVector x(3);
	IVector y(2);
	IMatrix A(2,3);

	x[0] = interval(9.0,11.0)/interval(10.0);
	x[1] = -x[0];
	x[2] = x[0];

	A[0][0] = interval(1);		A[0][1] = interval(2);		A[0][2] = interval(3);
	A[1][0] = -interval(3);		A[1][1] = -interval(2);		A[1][2] = -interval(1);

	y = A*x;

	cout << "y = " << y << endl << endl;		

	// Basic operations on matrixes
	IMatrix B = transpose(A);
	IMatrix C = (A+A)*B;

	cout << "C = " << C << endl << endl; 		

	IMatrix invC = gaussInverseMatrix(C);
	IVector v = gauss(C,y);

	cout << "solution of the equation y = Cv :    v = " << v << endl;
	cout << "invC = " << invC << endl << endl; 	
	cout << "C*invC " << C*invC << endl << endl;
	
	// Several computations at the same time
	cout << "performing several computations at once  " << transpose(gaussInverseMatrix(A*B)+C)*y << endl << endl; // ... etc. is ok.

	//  Useful commands for intervals and interval-vectors
	cout << "middle point of the IVector x " << midVector(x) << endl;
	cout << "checking whether one IVector is in the interior of another IVector: " << subsetInterior(midVector(x),x) << endl;

	interval a = x[0];
	cout << "an interval " << a << endl;
	cout << "middle of the interval " << a.mid() << endl;
	cout << "left side of the interval " << a.left() << endl;
	cout << "right side of the interval " << a.right() << endl;
	cout << "left side of the interval in doubles " << a.leftBound() << endl;
	cout << "right side of the interval in doubles " << a.rightBound() << endl;
}

// Below we have an example of computing eigenvalues and eigenvectors
// !!! this is only implemented for non-rigorous (DVector,DMatrix) computations !!! 
void eigenvectors()
{
	DMatrix D(2,2);
	D[0][0] = 5;    D[0][1] = 1;
  	D[1][0] = 3;    D[1][1] = 6;

  	DVector rE(2), iE(2);         	// real and imaginary parts of eigenvalues
  	DMatrix rVec(2,2), iVec(2,2); 	// real and imaginary parts of eigenvectors

  	computeEigenvaluesAndEigenvectors(D,rE,iE,rVec,iVec);

	cout << " D = " << D << endl;
	cout << " rE = " << rE << endl;
	cout << " iE = " << iE << endl;
	cout << " rVec = " << rVec << endl;
	cout << " iVec = " << iVec << endl;
	
	cout << "Jordan form for our matrix D: "<< gaussInverseMatrix(rVec)*D*rVec << endl;
}

int main(int argc, char* argv[])
{
  	cout.precision(5);
	try
	{
		linalgdemo();
		eigenvectors();
	}
	catch(exception& e)
  	{
    		cout << "\n\nException caught: "<< e.what() << endl;
  	}
  return 0;
} 
