#include <iostream>
using namespace std;

#include "capd/capdlib.h"
using namespace capd;
using namespace capd::alglib;
using namespace capd::matrixAlgorithms;

void plot(ofstream &file,const IVector &p)
{
    file << (p[0].rightBound()+p[0].leftBound())/2. <<" ";
    file << (p[1].rightBound()+p[1].leftBound())/2. <<" ";
    file << (p[0].rightBound()-p[0].leftBound())/2. <<" ";
    file << (p[1].rightBound()-p[1].leftBound())/2. <<endl;
}

void plot()
{
	// we declare p to be a sequence of N IVectors 
	int N = 64;	
	vector<IVector> p(N);	

	// we declare a rotation map
	IMap r = "par:phi;var:x,y;fun:x*cos(phi)-y*sin(phi),x*sin(phi)+y*cos(phi);";
	r.setParameter("phi",interval(1)/interval(10));

	// we define a single IVector x 
	IVector x(2);
	x[0] = interval(-1,1)/interval(1000);
	x[1] = x[0]+interval(1);

	// we assign x to be the first IVector in the sequence p
	p[0] = x;
	cout << "p[0] = " << p[0] << endl;

	// we fill in the sequence p
	for(int i=1;i<N;i++)
	{
		p[i] = r(p[i-1]);
		cout << "p[" << i <<"] = " << p[i] << endl;
	}

	// we write the sequence into a file for plotting 
	ofstream outdata; 
	outdata.open("NameOfFile.txt");
	outdata.precision(10);

	for(int i=0;i<N;i++)
	{
		plot(outdata,p[i]);
	}
	outdata.close();
	// the graph should be plotted in gnuplot by typing in:
	//
	//               plot "NameOfFile.txt" with boxxyerrorbars
	//
	// typing simply 
	//
	//		plot "NameOfFile.txt"
	//
	// will give a plot of middle points of our boxes
}
int main(int argc, char* argv[])
{
  	cout.precision(5);
	try
	{
		plot();				
	}
	catch(exception& e)
  	{
    		cout << "\n\nException caught: "<< e.what() << endl;
  	}
  return 0;
} 
